/*
$Id: seek.c,v 1.4 1995/10/16 16:44:43 ssmore Exp $

File Name : seek.c
Author    : ssmore

Entry Point(s) :

(1) passion_seek   prototype in passion_base.h

Modification History :

Module Description                                      Date      Author
------------------------------------------------------------------------------

Notes :

(1) Modifications to the entry points interface should be reflected in the 
    prototypes declarations.
*/

#ifndef PASSION_SEEK_C
#define PASSION_SEEK_C

#include <unistd.h>
#ifndef PIOFS
#include <sys/types.h>
#else
#include <piofs/piofs.h>
#include <piofs/piofs_ioctl.h>
#endif

#ifdef SP_PIOFS_MPI
#include <piofs/piofs.h>
#include <piofs/piofs_ioctl.h>
#endif

#include "pfile.h"

#define DEBUG

#ifdef DEBUG
#include <stdio.h>
#endif
/*
  Name : passion_seek

  Function : This function is used to seek inside a LAF. It always seeks
	to offset relative to the start of the LAF.

  Parameters : 
    Input :
			FileDescriptor - File descriptor of the LAF
			Offset         - The offset to which to seek
    Output :
      none
    Input/Output :
      none

  Return Value : 
    0, if successful
		-1, otherwise

  Algorithm :

		Seek the specified offset in the LAF

  Side Effects :
    none

  Shortcomings :
    (1) Can not take advantage of the facility in PIOFS which allows you
		to seek at gigabyte offsets.

  Modifications :

  Description                                       Date          Author
  ------------------------------------------------------------------------------
*/

int passion_seek(LAF_File fd, unsigned long int offset)
{
#ifdef PIOFS
piofs_lseek_t SeekStruct;
#endif
#ifdef SP_PIOFS_MPI
piofs_lseek_t SeekStruct;
#endif

#ifdef PIOFS
  SeekStruct.offset.h = 0;
  SeekStruct.offset.l = offset;
	SeekStruct.whence = SEEK_SET;
	if (piofsioctl(fd, PIOFS_LSEEK, &SeekStruct) != 0)
  {
    return -1;
  }
#else
#ifdef SP_PIOFS_MPI
  SeekStruct.offset.h = 0;
  SeekStruct.offset.l = offset;
	SeekStruct.whence = SEEK_SET;
	if (piofsioctl(fd, PIOFS_LSEEK, &SeekStruct) != 0)
  {
    return -1;
  }
#else
#ifdef HAVE_MPIO
  MPI_File_seek(fd, offset, MPI_SEEK_SET);
#else
  if (lseek(fd, (off_t) offset, SEEK_SET) == (off_t)-1)
  {
#ifdef DEBUG
fprintf(stderr,"passion_seek:lseek %d error.\n", offset);
#endif
    return -1;
  }
#endif
#endif
#endif

	return 0;

}

#endif  /* PASSION_SEEK_C */
